/*
   my.cc - compile/run my programs.

   (C) Copyright 2001, by Universiteit Leiden.

   $Revision: 1.1 $
   $Date: 2001/07/11 13:51:50 $
   $Source: n:/bf/prj/my/prg/src/RCS/MY.CC%v $
   %Author: M.J. Moene %

   Compiler        : Borland-C++ 3.0
   Operating system: MS-DOS 6.22
   Hardware system : PC XT, AT
*/

/*
 * my [-hc] [file | file.my | file.myc...]
 *
 * optionally compile and run specified file,
 * or just compile specified file only (-c option).
 *
 * file     - run .myc file, if available and up to date, otherwise
 *            compile, save .myc file and run code.
 * file.my  - compile specified .my file, save .myc file and run code.
 * file.myc - run specified .myc file.
 *
 * options:
 *
 * -h : show usage
 * -c : only compile, don't run code.
 * -v : verbose.
 * -vv: more verbose.
 */

/*
 * $Log: MY.CC%v $
 * Revision 1.1  2001/07/11  13:51:50  moene
 * Initial revision
 *
 */

#include "computer.h"			// for class MyComputer
#include    "error.h"			// for object myError
#include    "tools.h"			// for basename() etc.

#include    <stdio.h>			// for FILENAME_MAX
#include   <string.h>			// for strlwr()
#include <iostream.h>			// for class i/ostream

enum { E_OK, E_CL, E_PS, E_QT, E_IN, };	// program exit status

#if defined( __BORLANDC__ )
   extern unsigned _stklen = 32000;	// enlarge runtime stack

   extern int   opterr;		// getopt error value, int file getopt.c
   extern int   optind;		// getopt index, in file getopt.c
   extern char *optarg;		// getopt argument pointer, int file getopt.c
   extern int   getopt( int argc, char *argv[], char *optionS );
#else
   extern int   getopt( int argc, char * const *argv, char *optionS );
#endif


/*
 * usage - print version and usage information.
 */

int usage( int status, ostream& stream )
{
   stream <<
      "my  version 1.1, 11 July 2001  compile/run my programs.\n"
      "\n"
      "usage: my [-hcv] [file | file.my | file.myc ...]\n"
      "-h   : print this help\n"
      "-c   : compile only\n"
      "-d   : print debug information\n"
      "-v[v]: print progress messages\n";

   return status;
}


/*
 * main - compile / run specified .my/.myc program.
 */

int main(int argc, char *argv[])
{
   myError.setDebugLevel(0);

   int option;
   int optCompile = 0;
   int optVerbose = 0;

#if defined( __BORLANDC__ )
   myError.setProgramName( basename( strlwr( argv[0] ) ) );
#else
   myError.setProgramName( basename( argv[0] ) );
#endif

   /*
    * handle commandline options:
    */

   while( EOF != ( option = getopt( argc, argv, "hcdv" ) ) )
   {
      switch( option )
      {
	 case 'h':
	    return usage( E_OK , cout );
	    break;

	 case 'c':
	    optCompile = 1;
	    break;

	 case 'd':
	    myError.incDebugLevel();
	    break;

	 case 'v':
	    optVerbose++;
	    break;

	 default :
	    return usage( E_CL, cerr );
	    break;
      }
   }

   /*
    * process files specified on commandline:
    */

   if ( optind >= argc )				// no arguments: show usage
   {
      return usage( E_OK, cout );
   }

   else for ( ; optind < argc; ++optind )
   {
      MyComputer mc;

      /*
       * save and construct .my / .myc filenames:
       */

#if defined( __BORLANDC__ )
      char *inpname = strlwr( argv[ optind ] );
#else
      char *inpname = argv[ optind ];
#endif

      char   myname [ FILENAME_MAX ];
      char  mycname [ FILENAME_MAX ];

      strcat( strcpy(  myname, basename( inpname )), ".my" );
      strcat( strcpy( mycname, basename( inpname )), ".myc" );

      if ( optVerbose == 1 )
	 cerr << inpname << ":\n";

      /*
       * compile only (option -c):
       */

      if ( optCompile )
      {
	 if ( optVerbose >= 2 )
	    cerr << inpname << ": compiling to " << mycname << ":\n";

	 if ( mc.compile( myname, mycname ) )
	    return myError.error( E_PS );
      }

      /*
       * run (file):
       */

      else if ( 0 == strcmp( extname( inpname ), ""  ) )
      {
	 if ( optVerbose >= 2 )
	    cerr << inpname << ": optionally compile/run " << myname << ":\n";

	 if ( mc.compileIfRequired( myname, mycname )
	 ||   mc.runByteCode      (         mycname ) )
	    return myError.error( E_PS );
      }

      /*
       * compile and run (file.my):
       */

      else if ( 0 == strcmp( extname( inpname ), "my" ) )
      {
	 if ( optVerbose >= 2 )
	    cerr << inpname << ": compiling to/run from " << mycname << ":\n";

	 if ( mc.run( myname, mycname ) )
	    return myError.error( E_PS );
      }

      /*
       * run program (file.myc):
       */

      else if ( 0 == strcmp( extname( inpname ), "myc" ) )
      {
	 if ( optVerbose >= 2 )
	    cerr << inpname << ": run " << mycname << ":\n";

	 if ( mc.runByteCode( mycname ) )
	    return myError.error( E_PS );
      }

      /*
       * uncovered situation:
       */

      else cerr << "No compilation or run for '" << inpname << "'.\n";
   }

   return 0;
}


/*** End of file ***/
